from pathlib import Path

from pkglts.option_object import Option
from pkglts.version import __version__


class OptionGithub(Option):
    def version(self):
        return __version__

    def root_dir(self):
        return Path(__file__).parent

    def update_parameters(self, cfg):
        sec = dict(
            owner="{{ base.authors[0][0] }}",
            project="{{ base.pkgname }}",
            url="https://github.com/{{ github.owner }}/{{ github.project }}"
        )
        cfg[self._name] = sec

    def check(self, cfg):
        invalids = []
        project = cfg[self._name]['project']

        if not project:
            invalids.append('github.project')

        return invalids

    def require_option(self):
        return ['git']



import pytest
from pkglts.config_management import Config
from pkglts.option.base.option import OptionBase


@pytest.fixture()
def opt():
    return OptionBase('base')


def test_update_parameters(opt):
    cfg = {}
    opt.update_parameters(cfg)
    assert len(cfg['base']) == 4


def test_config_check_pkg_names(opt):
    for pkg in ('1mypkg', ' mypkg', '1', '1.mypkg',
                ' .mypkg', '.mypkg', 'None.mypkg', 'oa.o.mypkg'):
        cfg = Config(dict(base={'pkgname': pkg,
                                'namespace': None,
                                'namespace_method': "pkg_util",
                                'owner': 'moi',
                                'url': None}))
        assert 'base.pkgname' in opt.check(cfg)
        cfg = Config(dict(base={'pkgname': 'toto',
                                'namespace': pkg,
                                'namespace_method': "pkg_util",
                                'owner': 'moi',
                                'url': None}))
        assert 'base.namespace' in opt.check(cfg)


def test_require_option(opt):
    assert len(tuple(opt.require_option())) == 0


def test_require(opt):
    cfg = Config()
    opt.update_parameters(cfg)

    assert len(tuple(opt.require(cfg))) == 0



import pytest
from pkglts.config_management import Config
from pkglts.option.license.option import OptionLicense


@pytest.fixture()
def opt():
    return OptionLicense('license')


def test_update_parameters(opt):
    cfg = {}
    opt.update_parameters(cfg)
    assert len(cfg['license']) == 4


def test_config_check_license_name_exists(opt):
    cfg = Config(dict(license={'name': "", 'year': 2015,
                               'organization': "oa", 'project': "project"}))
    assert 'license.name' in opt.check(cfg)

    cfg = Config(dict(license={'name': "tugudu", 'year': 2015,
                               'organization': "oa", 'project': "project"}))
    assert 'license.name' in opt.check(cfg)


def test_require_option(opt):
    assert len(tuple(opt.require_option())) == 1


def test_require(opt):
    cfg = Config()
    opt.update_parameters(cfg)

    assert len(tuple(opt.require(cfg))) == 0



import pytest
from pkglts.config_management import Config
from pkglts.option.doc.option import OptionDoc


@pytest.fixture()
def opt():
    return OptionDoc('doc')


def test_update_parameters(opt):
    cfg = {}
    opt.update_parameters(cfg)
    assert len(cfg['doc']) == 3


def test_config_check_description_exists(opt):
    cfg = Config(dict(doc={'description': "mydescr", 'fmt': 'rst', 'keywords': []}))
    assert cfg['doc']['description'] == "mydescr"
    assert 'doc.description' not in opt.check(cfg)


def test_config_check_description_valid(opt):
    cfg = Config(dict(doc={'description': "", 'fmt': 'rst', 'keywords': []}))
    assert 'doc.description' in opt.check(cfg)


def test_config_check_keywords_exists(opt):
    cfg = Config(dict(doc={'description': "mydescr", 'fmt': 'rst', 'keywords': []}))
    assert len(cfg['doc']['keywords']) == 0
    assert 'doc.keywords' not in opt.check(cfg)


def test_config_check_fmt_valid(opt):
    cfg = Config(dict(doc={'description': "mydescr", 'fmt': 'walou', 'keywords': []}))
    assert 'doc.fmt' in opt.check(cfg)


def test_require_option(opt):
    assert len(tuple(opt.require_option())) == 1


def test_require(opt):
    cfg = Config(dict(base={}, doc={'fmt': 'rst'}))

    assert len(tuple(opt.require(cfg))) == 0

    cfg = Config(dict(base={}, doc={'fmt': 'md'}))
    assert len(tuple(opt.require(cfg))) == 1
